import javax.swing.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.Vector;

class ClientGUI extends JFrame implements ClientUI,
                                          GUI, 
                                          ActionListener
{
  RobotClient client;
  RobotChooser chooser;

  RobotSensorDisplay sensor_display;
  RobotSonarSensorDisplay sonar_sensor_display;

  char distance_increment;
  char angle_increment;

  private JButton scan_once;

  public ClientGUI(){
    super("Robot Client 1.0");
    init();
  }

  public void update() {
    Vector sensors = client.getRobot().getSensors();
    Vector simple_sensors = new Vector();
    SonarSensor sonar = null;
    for(int i=0; i<sensors.size(); i++)
      if(sensors.elementAt(i) instanceof SonarSensor)
        sonar = (SonarSensor)sensors.elementAt(i);
      else {
        simple_sensors.add(sensors.elementAt(i));
      }
    if(sensor_display!=null)
      sensor_display.refresh(simple_sensors);
    if(sonar_sensor_display!=null)
      sonar_sensor_display.refresh(sonar, client.getRobot().getInfo().getSonarDPI());
  }

  public void init(){
    client = null;
    chooser = null;
    sensor_display = null;
    sonar_sensor_display = null;
    distance_increment = (char)5;
    angle_increment = (char)45;
    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    this.getContentPane().setLayout(new BorderLayout());
    initMenuBar();
    initControlPanel();
    initSensorDisplay();
    pack();
  }

  private void initMenuBar(){
    JMenuBar menubar = new JMenuBar();

    JMenu action_menu = new JMenu("Action");
    action_menu.setMnemonic(KeyEvent.VK_A);
    JMenuItem choose_item = new JMenuItem("Choose Robot",KeyEvent.VK_C);
    choose_item.setAccelerator(KeyStroke.getKeyStroke(
                                 KeyEvent.VK_C, ActionEvent.ALT_MASK));
    choose_item.setActionCommand("choose robot");
    choose_item.addActionListener(this);
    action_menu.add(choose_item);
    menubar.add(action_menu);

    JMenu options_menu = new JMenu("Options");
    options_menu.setMnemonic(KeyEvent.VK_O);
    JCheckBoxMenuItem auto_scan = new JCheckBoxMenuItem("Auto Scan", true);
    auto_scan.setMnemonic(KeyEvent.VK_S);
    auto_scan.setAccelerator(KeyStroke.getKeyStroke(
                               KeyEvent.VK_S, ActionEvent.ALT_MASK));
    auto_scan.setActionCommand("auto scan");
    auto_scan.addActionListener(this);
    options_menu.add(auto_scan);
    menubar.add(options_menu);

    menubar.add(Box.createHorizontalGlue());

    JMenu help_menu = new JMenu("Help");
    help_menu.setMnemonic(KeyEvent.VK_H);
    JMenuItem about_item = new JMenuItem("About",KeyEvent.VK_A);
    about_item.setActionCommand("about");
    about_item.addActionListener(this);
    help_menu.add(about_item);
    menubar.add(help_menu);

    this.setJMenuBar(menubar);
  }

  private void initControlPanel() {
    String image_dir = System.getProperty("robot.path", "./") + "images/";
    System.out.println(image_dir);

    JPanel controls = new JPanel();
    controls.setLayout(new GridLayout(3,5));

    controls.add(Box.createGlue());

    JButton forward = new JButton(new ImageIcon(image_dir+"arrow_forward.gif")); 
    forward.setActionCommand("forward");
    forward.addActionListener(this);
    controls.add(forward);

    controls.add(Box.createGlue());
    controls.add(Box.createGlue());

    JToggleButton toggle_sense = new JToggleButton("Sense");
    toggle_sense.setActionCommand("toggle sense");
    toggle_sense.addActionListener(this);
    controls.add(toggle_sense);

    JButton turn_left = new JButton(new ImageIcon(image_dir+"arrow_turn_left.gif")); 
    turn_left.setActionCommand("turn left");
    turn_left.addActionListener(this);
    controls.add(turn_left);

    JButton stop = new JButton(new ImageIcon(image_dir+"stop.gif"));
    stop.setActionCommand("stop");
    stop.addActionListener(this);
    controls.add(stop);

    JButton turn_right = new JButton(new ImageIcon(image_dir+"arrow_turn_right.gif")); 
    turn_right.setActionCommand("turn right");
    turn_right.addActionListener(this);
    controls.add(turn_right);

    controls.add(Box.createGlue());

    scan_once = new JButton("Scan");
    scan_once.setActionCommand("scan once");
    scan_once.addActionListener(this);
    scan_once.setEnabled(false);
    controls.add(scan_once);

    controls.add(Box.createGlue());

    JButton backward = new JButton(new ImageIcon(image_dir+"arrow_backward.gif")); 
    backward.setActionCommand("backward");
    backward.addActionListener(this);
    controls.add(backward);

    controls.add(Box.createGlue());
    controls.add(Box.createGlue());
    controls.add(Box.createGlue());

    this.getContentPane().add(controls, BorderLayout.SOUTH);
  }

  private void initSensorDisplay() {
    sensor_display = new RobotSensorDisplay();
    this.getContentPane().add(sensor_display, BorderLayout.EAST);

    sonar_sensor_display = new RobotSonarSensorDisplay();
    this.getContentPane().add(sonar_sensor_display, BorderLayout.CENTER);
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("choose robot")) {
      if(chooser==null)
        chooser = new RobotChooser(this, client.getRobotList());
      //add window listener here
      chooser.setVisible(true);
    } else if(e.getActionCommand().equals("auto scan")) {
      Object auto_scan = e.getSource();
      if(auto_scan!=null && auto_scan instanceof JCheckBoxMenuItem) {
        boolean selected = ((JCheckBoxMenuItem)auto_scan).isSelected();
    	  client.setAutoScan(selected);
        scan_once.setEnabled(!selected);
      }
    } else if(e.getActionCommand().equals("toggle sense")) {
      Object toggle_sense = e.getSource();
      if(toggle_sense!=null && toggle_sense instanceof JToggleButton) {
        boolean selected = ((JToggleButton)toggle_sense).isSelected();
    	  client.setSense(selected);
      }
    } else if(e.getActionCommand().equals("scan once")) {
      client.scanOnce();
    } else if(e.getActionCommand().equals("forward")) {
      client.moveRobot(Movement.MOVE_FORWARD, distance_increment);
    } else if(e.getActionCommand().equals("backward")) {
      client.moveRobot(Movement.MOVE_BACKWARD, distance_increment);
    } else if(e.getActionCommand().equals("turn left")) {
      client.moveRobot(Movement.TURN_LEFT, angle_increment);
    } else if(e.getActionCommand().equals("turn right")) {
      client.moveRobot(Movement.TURN_RIGHT, angle_increment);
    } else if(e.getActionCommand().equals("stop")) {
      client.stopRobot();
    }
  }

  public void setClient(RobotClient client) {
    this.client = client;
  }

  public static void main (String args[])
  {
    ClientGUI gui = new ClientGUI();
    gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    gui.setVisible(true);
  }
}
