import java.util.Arrays;

public class LocalClientLink implements ClientLink {
  private char INVALID = (char)0;
  private char RESERVE = (char)255;
  private char PENDING = (char)254;

  private RobotServer server;
  private char[] client_robot_map;

  public LocalClientLink(RobotServer server) {
    this.server = server;
    client_robot_map = new char[255];
    Arrays.fill(client_robot_map, INVALID);
  }

  public RobotServer getServer() {return server;}

  public char getNextClientID() {
    for(int i=0; i<RobotServer.MAX_CONNECTIONS; i++)
      if(client_robot_map[i]==INVALID) {
        client_robot_map[i] = RESERVE;
        return (char)i;
      }
    return INVALID;
  }

  public char connect() {
    char client_id = getNextClientID();
    if(client_id>INVALID && client_id<RobotServer.MAX_CONNECTIONS) {
      client_robot_map[client_id] = PENDING;
      return client_id;
    } else
      return INVALID;
  }

  public boolean connectRobot(char client_id, char robot_id) {
    if(client_id>INVALID && client_id<RobotServer.MAX_CONNECTIONS &&
      client_robot_map[client_id]==PENDING) {
      client_robot_map[client_id] = robot_id;
      server.connectRobot(server.getRobot(robot_id));
      return true;
    } else
      return false;
  }

  public void disconnect(char client_id) {
    client_robot_map[client_id] = INVALID;
  }

  public void disconnectRobot(char client_id, char robot_id) {
    if(client_robot_map[client_id]==robot_id) {
      client_robot_map[client_id] = PENDING;
      server.disconnectRobot(server.getRobot(robot_id));
    }
  }
}
