import java.util.Vector;

public class LocalServerLink implements ServerLink {
  private char client_id;
  private LocalClientLink link;
  private boolean is_connected;

  public LocalServerLink(LocalClientLink link) {
    this.link = link;
    client_id = 0;
    is_connected = false;
  }

  public boolean connect() {
    client_id = link.connect();
    if(client_id!=0)
      is_connected = true;
    return is_connected;
  }

  public boolean connectRobot(char robot_id){
    return link.connectRobot(client_id, robot_id);
  }

  public void disconnect() {
    if(is_connected){
      link.disconnect(client_id);
      is_connected = false;
    }  
  }

  public void disconnectRobot(char robot_id) {
    link.disconnectRobot(client_id, robot_id);
  }

  public char[] getRobotIDs(boolean all_robots) {
    Vector robots = link.getServer().getRobots(all_robots);
    char[] ids = new char[robots.size()];
    for(int i=0; i<robots.size(); i++)
      ids[i]=((Robot)robots.elementAt(i)).getID();
    return ids;
  }

  public RobotInfo getRobotInfo(char robot_id) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      return robot.getInfo();
    } else
      return null;
  }

  public char[] getRobotSensorLayout(char robot_id) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      return robot.getSensorLayout();
    } else
      return null;
  }

  public char[] getRobotSensorData(char robot_id) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      return robot.getSensorData();
    } else
      return null;
  }

  public boolean setSense(char robot_id, boolean sense) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      link.getServer().setSense(client_id, robot_id, sense);
      //wait for response here
      return true;
    } else
      return false;
  }

  public boolean setAutoScan(char robot_id, boolean auto_scan) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      link.getServer().setAutoScan(client_id, robot_id, auto_scan);
      //wait for response here
      return true;
    } else
      return false;
  }

  public boolean scanOnce(char robot_id) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      link.getServer().scanOnce(client_id, robot_id);
      //wait for response here
      return true;
    } else
      return false;
  }

  public boolean moveRobot(char robot_id, char direction, char distance) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      link.getServer().moveRobot(client_id, robot_id, direction, distance);
      //wait for response here
      return true;
    } else
      return false;
  }

  public boolean stopRobot(char robot_id) {
    Robot robot = link.getServer().getRobot(robot_id);
    if(robot!=null) {
      link.getServer().stopRobot(client_id, robot_id);
      //wait for response here
      return true;
    } else
      return false;
  }
}
