import java.util.Vector;

public abstract class Robot implements RobotInterface {
  public static final char ANY_ROBOT_ID = 0;
  protected char id;
  protected char[] sensor_layout;
  protected Vector sensors;
  protected RobotInfo info;
  protected boolean has_sonar_sensor;
  protected boolean is_owned;
  protected boolean is_ready;
  protected boolean is_sensing;

  public Robot(char id) {
    this.id = id;
    this.sensor_layout = null;
    this.sensors = null;
    this.info = null;
    this.has_sonar_sensor = false;
    this.is_owned = false;
    this.is_ready = false;
    this.is_sensing = false;
  }

  public String toString() {
    if(info!=null) {
      String name = info.getName();
      if(name!=null && name!="")
        return name;
    }
    return "Robot #"+String.valueOf((int)id);
  }

  public char getID() {return  id;}
  public String getName() {
    if(info!=null) return info.getName();
    else return "";
  }
  public RobotInfo getInfo() {return info;}
  public char[] getSensorLayout() {return sensor_layout;}
  public Vector getSensors() {return sensors;}

  public abstract char[] getSensorData();

  public boolean hasSonarSensor() {return has_sonar_sensor;}
  public boolean isOwned() {return is_owned;}
  public boolean isReady() {return is_ready;}
  public boolean isSensing() {return is_sensing;}

  public void setInfo(RobotInfo info) {
    this.info = info;
    if(this.info!=null && this.sensors!=null)
      is_ready = true;
    else
      is_ready = false;
  }
  public void setOwned(boolean owned) {this.is_owned = owned;}
  public void setSensing(boolean sensing) {this.is_sensing = sensing;}

  public void setSensorLayout(char[] layout) {
    if(layout==null || layout.length%2 != 0){
      System.err.println("Error: Invalid Sensor Layout");
      return;
    }

    this.sensor_layout = layout;

    if(sensors==null)
      sensors = new Vector();
    else
      sensors.clear();

    for(int i=0; i<layout.length; i+=2){
      char type = layout[i];
      char location = layout[i+1];
      if(type==SensorType.SONAR_SENSOR) {
        sensors.add(new SonarSensor(location,info.getSonarDataSize()));
        has_sonar_sensor = true;
      } else
        sensors.add(new SimpleSensor(type,location));      
    }

    if(info!=null && sensors!=null)
      is_ready = true;
    else
      is_ready = false;
  }

  public abstract void setSensorData(char[] data);
  public abstract void setSonarData(char[] data);
}
