import java.util.Vector;

public abstract class RobotClient extends Thread
{
  protected ClientUI ui;
  protected ServerLink server_link;
  protected Robot robot;

  public boolean running;

  public RobotClient() {
    ui = null;
    server_link = null;
    robot = null;
  }

  public RobotClient(Robot robot) {
    ui = null;
    server_link = null;
    this.robot = robot;
  }

  public Vector getRobotList() {
    Vector robots = new Vector();
    char[] ids = server_link.getRobotIDs(false);
    Robot robot;
    for(int i=0; i<ids.length; i++){
      robot = new RobotHolder(ids[i]);
      robot.setInfo(server_link.getRobotInfo(robot.getID()));
      robot.setSensorLayout(server_link.getRobotSensorLayout(robot.getID()));
      robots.add(robot);
    }
    return robots;
  }

  public Robot getRobot() {
    return this.robot;
  }

  public void setUserInterface(ClientUI ui) {
    this.ui = ui;
    ui.setClient(this);
    ui.update();
  }

  public void setServerLink(ServerLink server_link) {
    this.server_link = server_link;
  }

  //this should take a parameter to specify where the remote server is
  public void setRemoteServer(int port) {
  }

  public boolean setSense(boolean sense) {
    if(robot!=null)
    	return server_link.setSense(robot.getID(), sense);
    else
    	return false;
  }

  public boolean setAutoScan(boolean auto_scan) {
    if(robot!=null)
    	return server_link.setAutoScan(robot.getID(), auto_scan);
    else
    	return false;
  }

  public boolean scanOnce() {
    if(robot!=null)
    	return server_link.scanOnce(robot.getID());
    else
    	return false;
  }

  public boolean moveRobot(char direction, char distance) {
    if(robot!=null)
    	return server_link.moveRobot(robot.getID(), direction, distance);
    else
    	return false;
  }

  public boolean stopRobot() {
    if(robot!=null)
    	return server_link.stopRobot(robot.getID());
    else
    	return false;
  }


  public void updateRobot() {
    if(robot!=null) {
      if(robot.getInfo()==null)
        robot.setInfo(server_link.getRobotInfo(robot.getID()));
      if(robot.getSensorLayout()==null)
        robot.setSensorLayout(server_link.getRobotSensorLayout(robot.getID()));
      robot.setSensorData(server_link.getRobotSensorData(robot.getID()));

      if(ui!=null) {
        ui.update();
      }
    }
  }
}
