import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.util.Vector;
import javax.swing.border.TitledBorder;
import java.awt.BorderLayout;
import java.awt.Dimension;

public class RobotSensorDisplay extends JPanel {
  protected RobotSensorDisplayTableModel model;

  public RobotSensorDisplay() {
    super();
    model = new RobotSensorDisplayTableModel();
    JTable table = new JTable(model);
    table.setPreferredScrollableViewportSize(new Dimension(250,300));
    table.getColumnModel().getColumn(0).setPreferredWidth(110);
    table.getColumnModel().getColumn(1).setPreferredWidth(110);
    table.getColumnModel().getColumn(2).setPreferredWidth(30);

    JScrollPane scroll_pane = new JScrollPane(table);

    this.add(scroll_pane, BorderLayout.CENTER);
  }

  public void refresh(Vector sensors) {
    model.refresh(sensors);
    repaint();
  }
}

class RobotSensorDisplayTableModel extends AbstractTableModel {
  private Vector sensors;

  //try just passing the relavent sensors
  public RobotSensorDisplayTableModel() {
    super();
    this.sensors = null;
  }

  public void refresh(Vector sensors) {
    this.sensors = sensors;
  }

  public int getRowCount() {
    if(sensors!=null)
      return sensors.size();
    else
      return 0;
  }

  public int getColumnCount() {
    return 3; //location, type, value
  }

  public String getColumnName(int column) {
    switch(column) {
      case 0:
        return new String("Location");
      case 1:
        return "Type";
      case 2:
        return "Value";
      default:
        return super.getColumnName(column);
    }
  }

  public Object getValueAt(int row, int column) {
    if(sensors!=null && row>=0 && row<sensors.size() && column>=0 && column<3) {
      SimpleSensor sensor = (SimpleSensor)sensors.elementAt(row);
      switch(column) {
        case 0:
          return RobotLocation.stringValue(sensor.location);
        case 1:
          return SensorType.stringValue(sensor.type);
        case 2:
          return String.valueOf((int)sensor.value);
        default:
          return "";
      }
    } else
      return "";
  }
}
