import javax.swing.*;
import java.awt.*;

public class RobotSonarSensorDisplay extends JPanel {
  protected SonarSensor sonar_sensor;
  protected int grid_width;
  protected int grid_height;
  protected int pixel_res;
  protected int dpi;

  public RobotSonarSensorDisplay() {
    this.grid_width = 512;
    this.grid_height = 256;
    this.dpi = 8; //resolution in dots per unit
    pixel_res = 1; //each dot is pixel_res pixels x pixel_res pixels
    this.setPreferredSize(new Dimension(grid_width,grid_height));
  }

  public void refresh(SonarSensor sonar_sensor, int dpi) {
    this.sonar_sensor = sonar_sensor;
    this.dpi = dpi;
    repaint();
  }

  protected void paintComponent(Graphics g) {
    super.paintComponent(g);

    if(sonar_sensor!=null) {
      Graphics2D g2d = (Graphics2D)g;
      Rectangle dot = new Rectangle(pixel_res, pixel_res);
      int pixel_width = grid_width*pixel_res;
      int pixel_height = grid_height*pixel_res;

      g2d.setColor(Color.WHITE);
      g2d.fillRect(0,0,pixel_width,pixel_height);
      g2d.setColor(Color.BLACK);
      g2d.translate(pixel_width/2+1, pixel_height-1);

      double increment = Math.PI/((double)(sonar_sensor.value.length - 1));
      double angle;
      double dist;
      int x, y;

      for(int i=0; i<sonar_sensor.value.length; i++) {
        angle = increment*(double)i;
        if(sonar_sensor.value[i]=='\0')
          continue;        
        dist = (double)sonar_sensor.value[i];
        //add 1 inch to allow for sonar offset
        dist+=dpi;
        x = (int)Math.floor(dist*Math.cos(angle)*(double)pixel_res);
        y = -(int)Math.floor(dist*Math.sin(angle)*(double)pixel_res);
        dot.setLocation(new Point(x,y));
        g2d.draw(dot);
      }
      
      //draw "robot"
      g2d.setColor(Color.BLUE);
      g2d.fill(new Rectangle(-4,-2,8,8));
    }
  }  
}

