import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

class ServerGUI extends JFrame implements UserInterface,
                                          GUI,
                                          ActionListener
{
  RobotServer server;
  Vector robots;
  RobotInfoViewer robot_info_viewer;
  RobotClient local_client;

  public ServerGUI() {
    super("Robot Server 1.0");
    this.server = server;
    init();
    pack();
  }

  public void setServer(RobotServer server) {
    this.server = server;
  }

  public void init(){
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.setSize(400,400);

    robots = new Vector();
    local_client = null;

    initContent();
  }

  private void initContent(){
    Container content = this.getContentPane();
    content.setLayout(new BorderLayout());

    JPanel robots_panel = new JPanel(new BorderLayout());

    robot_info_viewer = new RobotInfoViewer();
    content.add(robot_info_viewer, BorderLayout.CENTER);

    JButton start_client = new JButton("Start Client");
    start_client.setActionCommand("start client");
    start_client.addActionListener(this);
    content.add(start_client, BorderLayout.SOUTH);
  }

  public void update() {
    robot_info_viewer.refresh(robots);
  }

  public void addRobot(Robot robot) {
    robots.add(robot);
    update();
  }

  public void removeRobot(Robot robot) {
    robots.remove(robot);
    update();
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("start client")) {
      if(robot_info_viewer.getSelectedRobot()!=null) {
        server.startLocalClient(robot_info_viewer.getSelectedRobot());
      }
    }
  }
}

class MyCellRenderer extends JLabel implements ListCellRenderer {
  public Component getListCellRendererComponent(
    JList list,
    Object value,            // value to display
    int index,               // cell index
    boolean isSelected,      // is the cell selected
    boolean cellHasFocus)    // the list and the cell have the focus
  {
    String text = value.toString();

	  setFont(list.getFont());
    setOpaque(true);

    if(((Robot)value).isReady())
      text = "<html><b>"+text+"</b></html>";
    else
      text = "<html><font color=gray><i>"+text+"</i></font></html>";

    setText(text);
 
    if(((Robot)value).isReady() && !((Robot)value).isOwned())
      setEnabled(true);
    else
      setEnabled(false);

    if(isEnabled()) {
      if (isSelected) {
        setBackground(list.getSelectionBackground());
        setForeground(list.getSelectionForeground());
      } else {
        setBackground(list.getBackground());
        setForeground(list.getForeground());
      }
    }

    return this;
  }
}


